/*
 * 3D Morph Toolkit
 *
 * (c) 2000 Sylvain Tertois
 *
 *
 * MTPoints: class that store a group of points, with their colors
 */

#ifndef MTPOINTS_H
#define MTPOINTS_H

class MTVector;
class MTMatrix;
#include <Archivable.h>

struct rgb_color;

class BPoint;

class MTPoints : public BArchivable
{
public:
	MTPoints();
	MTPoints(BMessage*);
	MTPoints(const MTPoints&);
	MTPoints(const MTVector &x, const MTVector &y, const MTVector &z, const MTVector &r, const MTVector &g, const MTVector &b);
	MTPoints(unsigned int nPoints, const float *coordinates, const rgb_color *colors);
		// coordinates must point to an array of 3*nPoints floats, with the
		// coordinates in this order: x1,y1,z1,x2,y2,z2,x3,y3,z3,x4,y4,z4 etc...
	~MTPoints();
	
	// assignement functions
	MTPoints &SetTo(const MTPoints&);
	MTPoints &SetTo(const MTVector &x, const MTVector &y, const MTVector &z, const MTVector &r, const MTVector &g, const MTVector &b);
	MTPoints &SetTo(unsigned int nPoints, const float *coordinates, const rgb_color *colors);
		// coordinates must point to an array of 3*nPoints floats, with the
		// coordinates in this order: x1,y1,z1,x2,y2,z2,x3,y3,z3,x4,y4,z4 etc...
	inline MTPoints &operator=(const MTPoints &src)
		{ return SetTo(src); }
	
	// archive functions
	static BArchivable *Instantiate(BMessage*);
	virtual status_t Archive(BMessage*,bool deep=true) const;
	
	// functions to add more points
	MTPoints &Add(const MTVector &x,const MTVector &y,const MTVector &z,const MTVector &r,const MTVector &g,const MTVector &b);
	MTPoints &Add(const MTPoints&);
	inline MTPoints &operator&=(const MTPoints &src)
		{ return Add(src); }
	MTPoints operator&(const MTPoints&) const;
	
	// transfer the colors/coordinates from a group to another, or from vectors to a group
	// for the first two versions, the two groups must have the same number of points
	// returns false if the transfer could not be done
	bool SetColor(const MTPoints&);
	bool SetColor(const MTVector &r, const MTVector &g, const MTVector &b);
	/// these versions transfer one color on all the points
	bool SetColor(float r, float g, float b);
	bool SetColor(const rgb_color &color);
	/// these versions do a gradient between two colors
	bool SetColor(float r1, float g1, float b1, float r2, float g2, float b2);
	bool SetColor(const rgb_color &color1, const rgb_color &color2);
	
	bool SetCoordinates(const MTPoints&);
	bool SetCoordinates(const MTVector &x,const MTVector &y,const MTVector &z);
	
	// transformation functions
	MTPoints &Transform(const MTMatrix&);
	inline MTPoints &operator*=(const MTMatrix &matrix)
		{ return Transform(matrix); }
	MTPoints operator*(const MTMatrix&) const;

	// access to the number of values, and the buffer
	inline unsigned int NumPoints() const
		{ return m_NPoints; }
	inline const float *Coordinates() const
		{ return m_Coordinates; }
	inline const rgb_color *Colors() const
		{ return m_Colors; }
	
	// 3D-to-2D projection
	BPoint *Project(float depth,BPoint *proj) const;
	
	// morphing
	static MTPoints Morphing(MTPoints &p1, MTPoints &p2, float f);
		// does a morphing between the two objects p1 and p2
		// when f=0, the object p1 is returned
		// when f=1, the object p2 is returned
		// when f is between 0 and 1, an object beween p1 and p2 is returned
		// you can also try values under 0 or over 1 to have interesting effects ;-)
		
	// basic objects
	static MTPoints Segment(float x1,float y1, float z1, float x2, float y2, float z2, unsigned int nPoints);
	static MTPoints Circle(unsigned int nPoints);
		// circle in the x-y plane, centered at (0,0,0) with a 1 radius
	
private:
	unsigned int m_NPoints;
	float *m_Coordinates;
	rgb_color *m_Colors;
};

#endif //MTPOINTS_H
